using System;
using UnityEngine;
using UnityEngine.Events;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Threading;
using MVR;
using MeshVR;
using System.Linq;


namespace Stopper {
	public class ShoeHelper : MVRScript {
		DAZCharacterSelector characterSelector;
		
		List<ConfigurableJoint> toeJoints = new List<ConfigurableJoint>();
		
		string selectedClothingUid;
		DAZClothingItem shoeClothingItem;
		
		Vector3 shoeColliderDimensions = new Vector3(0.01f, 0.01f, 0.15f);
		Vector3 shoeColliderCenter = new Vector3(0.0f, 0.0f, 0.075f);
		Vector3 shoeColliderRotation = new Vector3(20.0f, 0f, 0.0f);
		
		float shoeFootDriveTarget = -45f;
		float shoeToeDriveTarget = 45f;
		
		public override void Init() {
			characterSelector = containingAtom.GetComponentInChildren<DAZCharacterSelector>();
			
			Transform physicsModel = containingAtom.gameObject.transform.Find("rescale2/PhysicsModel");
			foreach(Rigidbody rb in physicsModel.GetComponentsInChildren<Rigidbody>()){
				if(rb.name.Contains("SmallToe") || rb.name.Contains("BigToe") ){
					// There may be multple joints, but this seems to get the right one
					toeJoints.Add(rb.GetComponent<ConfigurableJoint>());
				}
			}
			
			JSONStorableStringChooser clothingChooser = new JSONStorableStringChooser(
					"High Heel Clothing", 
					containingAtom.GetComponentsInChildren<DAZClothingItem>().Select(c => c.uid).ToList(),
					null, 
					"High Heel Clothing", 
					(string uid) => {
						if(shoeClothingItem != null){
							DisconnectClothingItem(shoeClothingItem);
						}
						
						shoeClothingItem = characterSelector.GetClothingItem(uid);

						
						if(shoeClothingItem != null && !shoeClothingItem.name.Contains("Harli")) {
							SyncShoeSettings(shoeClothingItem);
							ConnectClothingItem(shoeClothingItem);
						} else {
							SuperController.LogMessage($"Could not find Clothing {selectedClothingUid}");	
						}
					});
					
			RegisterStringChooser(clothingChooser);
			var selectShoePopup = CreateScrollablePopup(clothingChooser, false);
			selectShoePopup.popup.onOpenPopupHandlers += () => {
				clothingChooser.choices = containingAtom.GetComponentsInChildren<DAZClothingItem>().Select(c => c.uid).ToList();
			};
			
			
			AddSlider("Heel Collider Length", 0.15f, 0f, 0.5f, (float val) => {
				shoeColliderDimensions.z = val;
				SyncShoeSettings(shoeClothingItem);
			}, false);
			
			AddSlider("Heel Collider Wifth", 0.01f, 0f, 0.1f, (float val) => {
				shoeColliderDimensions.x = val;
				shoeColliderDimensions.y = val;
				SyncShoeSettings(shoeClothingItem);
			}, false);
			
			AddSlider("Heel Collider Rotation", 20f, -90f, 90f, (float val) => {
				shoeColliderDimensions.x = val;
				SyncShoeSettings(shoeClothingItem);
			}, false);
			
			AddSlider("Toe Rotation Target", shoeToeDriveTarget, -75f, 75f, (float val) => {
				shoeToeDriveTarget = val;
				SyncShoeSettings(shoeClothingItem);
			}, false);
			
			AddSlider("Foot Rotation Target", shoeFootDriveTarget, -65f, 65f, (float val) => {
				shoeFootDriveTarget = val;
				SyncShoeSettings(shoeClothingItem);
			}, false);
			

			
			AddSlider("Toe Stiffness", 10f, 0f, 2000f, (float val) => {
				foreach(var joint in toeJoints){
					var slerpDrive = joint.slerpDrive;
					slerpDrive.positionSpring = val;
					joint.slerpDrive = slerpDrive;
				}
			}, true);
			
			AddSlider("Toe Damper", 10f, 0f, 2000f, (float val) => {
				foreach(var joint in toeJoints){
					var slerpDrive = joint.slerpDrive;
					slerpDrive.positionDamper = val;
					joint.slerpDrive = slerpDrive;
				}
			}, true);
			
			AddSlider("Toe Max Force", 10f, 0f, 2000f, (float val) => {
				foreach(var joint in toeJoints){
					var slerpDrive = joint.slerpDrive;
					slerpDrive.maximumForce = val;
					joint.slerpDrive = slerpDrive;
				}
			}, true);
			
			
		}
		
		private void SyncShoeSettings(DAZClothingItem shoeClothingItem){
			shoeClothingItem.enabled = false;
			shoeClothingItem.colliderTypeLeft = DAZClothingItem.ColliderType.Shoe;
			shoeClothingItem.colliderTypeRight = DAZClothingItem.ColliderType.Shoe;
			
			shoeClothingItem.driveXAngleTargetController1Type = DAZClothingItem.ControllerType.RightToe;
			shoeClothingItem.driveXAngleTargetController2Type = DAZClothingItem.ControllerType.LeftToe;
			
			shoeClothingItem.drive2XAngleTargetController1Type = DAZClothingItem.ControllerType.RightFoot;
			shoeClothingItem.drive2XAngleTargetController2Type = DAZClothingItem.ControllerType.LeftFoot;
			
			shoeClothingItem.colliderDimensions = shoeColliderDimensions;
			
			shoeClothingItem.colliderRightCenter = shoeColliderCenter;
			shoeClothingItem.colliderLeftCenter = shoeColliderCenter;
			
			shoeClothingItem.colliderRightRotation = shoeColliderRotation;
			shoeClothingItem.colliderLeftRotation = shoeColliderRotation;
			
			shoeClothingItem.driveXAngleTarget = shoeToeDriveTarget;
			shoeClothingItem.drive2XAngleTarget = shoeFootDriveTarget;
			shoeClothingItem.enabled = true;
		}
		
		private string PrintVector(Vector3 vector){
			return $"{vector.x} , {vector.y} , {vector.z}";
		}
		
		// Copied from DAZCharacterSelector
		private void ConnectClothingItem(DAZClothingItem dci)
		{
			shoeClothingItem.enabled = false;
			DAZClothingItem.ColliderType colliderTypeLeft = dci.colliderTypeLeft;
			if (colliderTypeLeft != DAZClothingItem.ColliderType.None)
			{
				if (colliderTypeLeft == DAZClothingItem.ColliderType.Shoe)
				{
					dci.colliderLeft = characterSelector.leftShoeCollider;
				}
			}
			else
			{
				dci.colliderLeft = null;
			}
			DAZClothingItem.ColliderType colliderTypeRight = dci.colliderTypeRight;
			if (colliderTypeRight != DAZClothingItem.ColliderType.None)
			{
				if (colliderTypeRight == DAZClothingItem.ColliderType.Shoe)
				{
					dci.colliderRight = characterSelector.rightShoeCollider;
				}
			}
			else
			{
				dci.colliderRight = null;
			}
			switch (dci.driveXAngleTargetController1Type)
			{
			case DAZClothingItem.ControllerType.None:
				dci.driveXAngleTargetController1 = null;
				break;
			case DAZClothingItem.ControllerType.LeftFoot:
				dci.driveXAngleTargetController1 = characterSelector.leftFootController;
				break;
			case DAZClothingItem.ControllerType.RightFoot:
				dci.driveXAngleTargetController1 = characterSelector.rightFootController;
				break;
			case DAZClothingItem.ControllerType.LeftToe:
				dci.driveXAngleTargetController1 = characterSelector.leftToeController;
				break;
			case DAZClothingItem.ControllerType.RightToe:
				dci.driveXAngleTargetController1 = characterSelector.rightToeController;
				break;
			}
			switch (dci.driveXAngleTargetController2Type)
			{
			case DAZClothingItem.ControllerType.None:
				dci.driveXAngleTargetController2 = null;
				break;
			case DAZClothingItem.ControllerType.LeftFoot:
				dci.driveXAngleTargetController2 = characterSelector.leftFootController;
				break;
			case DAZClothingItem.ControllerType.RightFoot:
				dci.driveXAngleTargetController2 = characterSelector.rightFootController;
				break;
			case DAZClothingItem.ControllerType.LeftToe:
				dci.driveXAngleTargetController2 = characterSelector.leftToeController;
				break;
			case DAZClothingItem.ControllerType.RightToe:
				dci.driveXAngleTargetController2 = characterSelector.rightToeController;
				break;
			}
			switch (dci.drive2XAngleTargetController1Type)
			{
			case DAZClothingItem.ControllerType.None:
				dci.drive2XAngleTargetController1 = null;
				break;
			case DAZClothingItem.ControllerType.LeftFoot:
				dci.drive2XAngleTargetController1 = characterSelector.leftFootController;
				break;
			case DAZClothingItem.ControllerType.RightFoot:
				dci.drive2XAngleTargetController1 = characterSelector.rightFootController;
				break;
			case DAZClothingItem.ControllerType.LeftToe:
				dci.drive2XAngleTargetController1 = characterSelector.leftToeController;
				break;
			case DAZClothingItem.ControllerType.RightToe:
				dci.drive2XAngleTargetController1 = characterSelector.rightToeController;
				break;
			}
			switch (dci.drive2XAngleTargetController2Type)
			{
			case DAZClothingItem.ControllerType.None:
				dci.drive2XAngleTargetController2 = null;
				break;
			case DAZClothingItem.ControllerType.LeftFoot:
				dci.drive2XAngleTargetController2 = characterSelector.leftFootController;
				break;
			case DAZClothingItem.ControllerType.RightFoot:
				dci.drive2XAngleTargetController2 = characterSelector.rightFootController;
				break;
			case DAZClothingItem.ControllerType.LeftToe:
				dci.drive2XAngleTargetController2 = characterSelector.leftToeController;
				break;
			case DAZClothingItem.ControllerType.RightToe:
				dci.drive2XAngleTargetController2 = characterSelector.rightToeController;
				break;
			}
			shoeClothingItem.enabled = true;
		}
		
		private void DisconnectClothingItem(DAZClothingItem shoeClothingItem){
			shoeClothingItem.enabled = false;
			shoeClothingItem.colliderTypeLeft = DAZClothingItem.ColliderType.None;
			shoeClothingItem.colliderTypeRight = DAZClothingItem.ColliderType.None;
			
			shoeClothingItem.driveXAngleTargetController1Type = DAZClothingItem.ControllerType.None;
			shoeClothingItem.driveXAngleTargetController2Type = DAZClothingItem.ControllerType.None;
			
			shoeClothingItem.drive2XAngleTargetController1Type = DAZClothingItem.ControllerType.None;
			shoeClothingItem.drive2XAngleTargetController2Type = DAZClothingItem.ControllerType.None;
			
			ConnectClothingItem(shoeClothingItem);
			shoeClothingItem.enabled = true;
		}
		
		public JSONStorableFloat AddSlider(string name, float defaultVal, float min, float max, UnityAction<float> onChange = null, bool rightSide = false)
		{
			JSONStorableFloat storableFloat = new JSONStorableFloat(name, defaultVal, min, max, true, true);
			storableFloat.storeType = JSONStorableParam.StoreType.Full;
			UIDynamicSlider uiSlider = this.CreateSlider(storableFloat, rightSide);
			uiSlider.rangeAdjustEnabled = true;
			RegisterFloat(storableFloat);
			
			if(onChange != null){
				uiSlider.slider.onValueChanged.AddListener(onChange);
			}
			
			return storableFloat;
		}
		
		private void Start(){
			
		}
	
			
		private void OnDisable()
		{
			if(shoeClothingItem != null){
				DisconnectClothingItem(shoeClothingItem);
			}
		}
		
		private void OnEnable()
		{
			if(shoeClothingItem != null){
				SyncShoeSettings(shoeClothingItem);
				ConnectClothingItem(shoeClothingItem);
			}
		}
		
		private void OnDestroy(){
		}
	}
}
